package net.sf.uctool.mojo;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import net.sf.xsltmp.FinalizeMojo;
import net.sf.xsltmp.FromManyBase;
import net.sf.xsltmp.InitializeMojo;
import net.sf.xsltmp.ManyToDynamicMojo;
import net.sf.xsltmp.ManyToOneMojo;

import org.apache.maven.model.Resource;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.project.MavenProject;
import org.codehaus.plexus.archiver.manager.ArchiverManager;
import org.codehaus.plexus.util.StringUtils;

/**
 * Generate use case documentation.
 * 
 * @goal use-cases
 * @phase generate-resources
 * @requiresDependencyResolution compile
 */
public class UseCasesMojo extends AbstractMojo {

	/**
	 * The directory holding the use case XML source files.
	 * 
	 * @parameter default-value="${project.basedir}/src/main/ucs/"
	 * @required
	 */
	private File useCasesDirectory;

	/**
	 * Which files from the use cases directory tree should be included in the
	 * generation.
	 * <p>
	 * Specifies a file-set of source files in the comma- or space-separated
	 * list of patterns of files.
	 * 
	 * @parameter default-value="**\/*.xml"
	 * @required
	 */
	private String useCasesInclude;

	/**
	 * The output directory. The HTML use case documentation is generated there.
	 * 
	 * @parameter default-value="${project.build.directory}/site/ucs/"
	 * @required
	 */
	private File outputDirectory;

	/**
	 * Read the source files (XSL templates and XML transformation sources)
	 * using this encoding.
	 * 
	 * @parameter expression="${project.build.sourceEncoding}"
	 */
	private String sourceEncoding;

	/**
	 * Location of file to hold merged XML use case source files.
	 * 
	 * @parameter default-value=
	 *            "${project.build.outputDirectory}/net/sf/uctool/merged.xml"
	 * @required
	 */
	private File mergedFile;

	/**
	 * Set language to load the resource bundle with fixed texts translations.
	 * When not specified, the default locale language is used.
	 * 
	 * @parameter
	 */
	private String bundleLanguage;

	/**
	 * Set name to display on the start HTML page.
	 * 
	 * @parameter default-value= "${project.name}"
	 */
	private String useCasesName;

	/**
	 * Set version to display on the start HTML page.
	 * 
	 * @parameter default-value= "${project.version}"
	 */
	private String useCasesVersion;

	/**
	 * The Maven project model.
	 * 
	 * @parameter expression="${project}"
	 * @required
	 * @readonly
	 */
	private MavenProject project;

	/**
	 * To look up Archiver/UnArchiver implementations.
	 * 
	 * @component
	 */
	private ArchiverManager archiverManager;

	public void execute() throws MojoExecutionException, MojoFailureException {
		// execute individual phases:
		merge();
		intialize();
		actors();
		actorIndex();
		data();
		dataIndex();
		useCases();
		useCaseIndex();
		actorGoalList();
		entryPointList();
		unusedDataList();
		summaryIndex();
		singlePage();
		index();
		finalize_();
		addUcsDirToResources();
	}

	// phase methods:

	private void merge() throws MojoExecutionException, MojoFailureException {
		ManyToOneMojo mojo = prepareManyToOne("Merge source files", "Merge");
		mojo.setDestFile(mergedFile);
		mojo.prepare();
		if (!"".equals(mojo.getParameters().get("source-file-names"))) {
			mojo.setForce(true);
		}
		mojo.execute();
	}

	private void intialize() throws MojoExecutionException,
			MojoFailureException {
		logPhase("Initialize XSLT generator Maven plugin");
		InitializeMojo mojo = new InitializeMojo();
		mojo.setProject(project);
		mojo.setLog(getLog());
		mojo.execute();
	}

	private void actors() throws MojoExecutionException, MojoFailureException {
		generateManyToDynamic("actors", "Actor", "actor");
	}

	private void actorIndex() throws MojoExecutionException,
			MojoFailureException {
		generateManyToOne("actor index", "ActorIndex", "actor/index.html");
	}

	private void data() throws MojoExecutionException, MojoFailureException {
		generateManyToDynamic("data", "Data", "data");
	}

	private void dataIndex() throws MojoExecutionException,
			MojoFailureException {
		generateManyToOne("data index", "DataIndex", "data/index.html");
	}

	private void useCases() throws MojoExecutionException, MojoFailureException {
		generateManyToDynamic("use cases", "UseCase", "uc");
	}

	private void useCaseIndex() throws MojoExecutionException,
			MojoFailureException {
		generateManyToOne("use case index", "UseCaseIndex", "uc/index.html");
	}

	private void actorGoalList() throws MojoExecutionException,
			MojoFailureException {
		generateManyToOne("actor-goal list", "ActorGoalList",
				"summary/actor-goal-list.html");
	}

	private void entryPointList() throws MojoExecutionException,
			MojoFailureException {
		generateManyToOne("entry point list", "EntryPointList",
				"summary/entry-point-list.html");
	}

	private void unusedDataList() throws MojoExecutionException,
			MojoFailureException {
		generateManyToOne("unused data list", "UnusedDataList",
				"summary/unused-data-list.html");
	}

	private void summaryIndex() throws MojoExecutionException,
			MojoFailureException {
		generateManyToOne("summary index", "SummaryIndex", "summary/index.html");
	}

	private void singlePage() throws MojoExecutionException,
			MojoFailureException {
		generateManyToOne("single page", "SinglePage", "single/index.html",
				true);
	}

	private void index() throws MojoExecutionException, MojoFailureException {
		generateManyToOne("index", "Index", "index.html", true);
	}

	private void finalize_() throws MojoExecutionException,
			MojoFailureException {
		logPhase("Finalize XSLT generator Maven plugin");
		FinalizeMojo mojo = new FinalizeMojo();
		mojo.setProject(project);
		mojo.setLog(getLog());
		mojo.execute();
	}

	private void addUcsDirToResources() {
		Resource resource = new Resource();
		resource.setDirectory(outputDirectory.getAbsolutePath());
		resource.setTargetPath(outputDirectory.getName());
		getProject().addResource(resource);
		getLog().info("... Added ucs cases directory to project resources.");
	}

	// helper methods:

	private void initMojo(FromManyBase mojo, String template) {
		mojo.setProject(project);
		mojo.setArchiverManager(archiverManager);
		mojo.setLog(getLog());
		mojo.setSrcDir(useCasesDirectory);
		mojo.setSrcIncludes(useCasesInclude);
		mojo.setSourceEncoding(getSourceEncoding());
		mojo.setFilter("net.sf.xsltmp.filter.TranslatingFilter");
		Map filterParameters = new HashMap();
		filterParameters
				.put("bundle", "generator/uctool/translations/Resource");
		filterParameters.put("bundleEncoding", getSourceEncoding());
		if (null != bundleLanguage)
			filterParameters.put("bundleLanguage", bundleLanguage);
		mojo.setFilterParameters(filterParameters);
		mojo.setXslTemplate(template);
	}

	private Map mojoParameters;

	public Map getMojoParameters() {
		if (null == mojoParameters) {
			mojoParameters = new HashMap();
			mojoParameters
					.put("merged-file-name", mergedFile.getAbsolutePath());
		}
		return mojoParameters;
	}

	private void logPhase(String label) {
		getLog().info("... " + label + ":");
	}

	private void generateManyToOne(String phase, String template, String output)
			throws MojoExecutionException, MojoFailureException {
		generateManyToOne(phase, template, output, false);
	}

	private void generateManyToOne(String phase, String template,
			String output, boolean passParams) throws MojoExecutionException,
			MojoFailureException {
		ManyToOneMojo mojo = prepareManyToOne("Generate " + phase, template);
		mojo.setSrcFile(mergedFile);
		mojo.setDestFile(new File(outputDirectory, output));
		if (passParams) {
			Map parameters = new HashMap();
			parameters.put("ucs-name", useCasesName);
			parameters.put("ucs-version", useCasesVersion);
			mojo.setParameters(parameters);
		}
		mojo.execute();
	}

	private ManyToOneMojo prepareManyToOne(String phase, String template)
			throws MojoExecutionException, MojoFailureException {
		logPhase(phase);
		ManyToOneMojo mojo = new ManyToOneMojo();
		initMojo(mojo, "generator/uctool/" + template + ".xsl");
		return mojo;
	}

	private void generateManyToDynamic(String phase, String template,
			String destDir) throws MojoExecutionException, MojoFailureException {
		logPhase("Generate " + phase);
		ManyToDynamicMojo mojo = new ManyToDynamicMojo();
		initMojo(mojo, "generator/uctool/" + template + ".xsl");
		mojo.setDestDir(new File(outputDirectory, destDir));
		mojo.setParameters(getMojoParameters());
		mojo.execute();
	}

	// getters, setters:

	public String getSourceEncoding() {
		if (StringUtils.isEmpty(sourceEncoding)) {
			sourceEncoding = System.getProperty("file.encoding");
			getLog().warn(
					"Source encoding has not been set, "
							+ "using platform encoding " + sourceEncoding
							+ ", i.e. build is platform dependent!");
		}
		return sourceEncoding;
	}

	public MavenProject getProject() {
		return project;
	}

}
